local _, L = ...
local mediaFolder = "Interface\\AddOns\\".._.."\\Media\\"
local AceGUI = LibStub("AceGUI-3.0")
local Media = LibStub("LibSharedMedia-3.0")

npcInfoPanelMixin = {}

local function GetValueForKey(t, key)
  for k, v in pairs(t) do
    if k == key then return v end
  end
  return nil
end

local function GetKeyForValue(t, value)
  for k, v in pairs(t) do
    if v == value then return k end
  end
  return nil
end

local function CreateHeaderPanel(name)
	local offset = 12
	local button = CreateFrame("Button", name, myNPCInfoPanelFrameBase)
	button:SetSize(360, 30)				
	
	button.shadow = CreateFrame("Frame", nil, button, "BackdropTemplate")
	button.shadow:SetFrameLevel(button:GetFrameLevel())
	button.shadow:ClearAllPoints()
	button.shadow:SetPoint("TOPLEFT", button, "TOPLEFT", -offset, offset)
	button.shadow:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", offset, -offset)
	button.shadow:SetBackdrop({bgFile = mediaFolder..'Backdrop_Talkbox.blp', edgeFile = mediaFolder..'Edge_Talkbox_BG.blp',	edgeSize = offset, insets = { left = offset, right = offset, top = offset, bottom = offset }})
	button.shadow:SetBackdropBorderColor(0, 0, 0, 1)
	button.shadow:SetScript("OnEnter", PanelOnEnter)
	button.shadow:SetScript("OnLeave", PanelOnLeave)
	button.shadow:Show()

	button.DescriptionGroup = CreateFrame("Button", nil, myNPCInfoPanelFrameBase)		
	button.DescriptionGroup:SetAllPoints(button)
	button.DescriptionGroup:SetParent(button)
	button.DescriptionGroup:Show()
	
	button.npcName = button.DescriptionGroup:CreateFontString(nil, "OVERLAY")
	button.npcName:SetFont(STANDARD_TEXT_FONT, 18, outline or "OUTLINE")
	button.npcName:SetPoint("LEFT", button.DescriptionGroup, "LEFT", 4, 0)
	button.npcName:SetJustifyH("LEFT")
	button.npcName:SetText(nil)
	button.npcName:Show()
		
	button.WidgetGroup = AceGUI:Create("SimpleGroup")
	button.WidgetGroup.content:SetAllPoints(button)
	button.WidgetGroup.content:SetParent(button)
	button.WidgetGroup.content:Hide()
	button.WidgetGroup:SetLayout("Flow")
	
	button.DescEditBox = AceGUI:Create("EditBox")
	button.DescEditBox:SetWidth(180)
	button.DescEditBox.editbox:SetScript("OnEnter",function()
		local text = ""		
		for i, texture in pairs(ICON_LIST) do
			text = text .. i .. texture .."14:14|t "
		end
	
		GameTooltip:SetOwner(button.DescEditBox.editbox, "ANCHOR_BOTTOMLEFT", 0, 0)
		GameTooltip:AddLine(text, 1, 1, 1)
		GameTooltip:Show()
	end)
	button.DescEditBox.editbox:SetScript("OnLeave",function()
		GameTooltip:Hide()
	end)
	
	button.WidgetGroup:AddChild(button.DescEditBox)
		
	button.Colored = AceGUI:Create("CheckBox")
	button.Colored:SetWidth(80)
	button.Colored:SetLabel("Color")
	button.Colored.text:SetJustifyH("LEFT")
	button.Colored.text:SetFontObject("GameFontNormalSmall")
	button.WidgetGroup:AddChild(button.Colored)
		
	button.Color = AceGUI:Create("ColorPicker")
	button.Color:SetWidth(80)
	button.Color:SetLabel("Color")
	button.Color.text:SetJustifyH("LEFT")
	button.Color:SetHasAlpha(true)
	button.Color.text:SetFontObject("GameFontNormalSmall")
	button.WidgetGroup:AddChild(button.Color)
	
	button.WidgetGroup.content:Hide()
			
	button:Show()
	return button
end

local function CreateSpellPanel(name)
	local offset = 12
	local button = CreateFrame("Button", name, myNPCInfoPanelFrameBase)
	--button:SetScript("OnEnter", PanelOnEnter)
	--button:SetScript("OnLeave", PanelOnLeave)
	--button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	
	button.shadow = CreateFrame("Frame", nil, button, "BackdropTemplate")
	button.shadow:SetFrameLevel(button:GetFrameLevel())
	button.shadow:ClearAllPoints()
	button.shadow:SetPoint("TOPLEFT", button, "TOPLEFT", -offset, offset)
	button.shadow:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", offset, -offset)
	button.shadow:SetBackdrop({bgFile = mediaFolder..'Backdrop_Talkbox.blp', edgeFile = mediaFolder..'Edge_Talkbox_BG.blp',	edgeSize = offset, insets = { left = offset, right = offset, top = offset, bottom = offset }})
	button.shadow:SetBackdropBorderColor(0, 0, 0, 1)
	button.shadow:SetScript("OnEnter", PanelOnEnter)
	button.shadow:SetScript("OnLeave", PanelOnLeave)
	button.shadow:Show()
	
	button.DescriptionGroup = CreateFrame("Button", nil, myNPCInfoPanelFrameBase)		
	button.DescriptionGroup:SetAllPoints(button)
	button.DescriptionGroup:SetParent(button)
	button.DescriptionGroup:Show()
	
	button.icon = button.DescriptionGroup:CreateTexture(nil, "OVERLAY")
	button.icon:SetPoint("TOPLEFT", button.DescriptionGroup, "TOPLEFT", 2, -2)
	button.icon:SetPoint("BOTTOMRIGHT", button.DescriptionGroup, "BOTTOMLEFT", 75, 2)
	button.icon:SetTexCoord(.08, .92, 0.08, 0.92)
	button.icon:Show()
	
	button.desc = button.DescriptionGroup:CreateFontString(nil, "OVERLAY")
	button.desc:SetFont(STANDARD_TEXT_FONT, 11, outline or "OUTLINE")
	button.desc:SetPoint("TOPLEFT", button.icon, "TOPRIGHT", 4, 0)
	button.desc:SetPoint("BOTTOMRIGHT", button.DescriptionGroup, "BOTTOMRIGHT", 0, 0)
	button.desc:SetJustifyH("LEFT")
	button.desc:SetJustifyV("TOP")
	button.desc:SetText(nil)
	button.desc:Show()
	
	
	button.WidgetGroup = AceGUI:Create("SimpleGroup")
	button.WidgetGroup.content:SetAllPoints(button)
	button.WidgetGroup.content:SetParent(button)
	button.WidgetGroup.content:Hide()
	button.WidgetGroup:SetLayout("Flow")

	button.Slider = AceGUI:Create("Slider")
	button.Slider:SetSliderValues(1, 2, .1)
	button.Slider:SetLabel("Castbar scale")
	button.Slider.label:SetJustifyH("LEFT")
	button.Slider.label:SetFontObject("GameFontNormalSmall")
	button.Slider:SetWidth(135)
	button.Slider.editbox:SetWidth(40)
	button.Slider:SetIsPercent(true)
	button.WidgetGroup:AddChild(button.Slider)
	
	button.SoundDropDown = AceGUI:Create("Dropdown")
	button.SoundDropDown:SetWidth(135)
	button.SoundDropDown:SetLabel("Spell sound")
	button.SoundDropDown.text:SetJustifyH("LEFT")
	button.SoundDropDown:SetList(Media:List("sound"))
	button.WidgetGroup:AddChild(button.SoundDropDown)
	
	button.PredictedBox = AceGUI:Create("EditBox")
	button.PredictedBox:SetWidth(90)
	button.PredictedBox:SetLabel("Predicted time")
	button.WidgetGroup:AddChild(button.PredictedBox)
	
	button.TargetWarning = AceGUI:Create("CheckBox")
	button.TargetWarning:SetWidth(65)
	button.TargetWarning:SetLabel("Target")
	button.TargetWarning.text:SetJustifyH("LEFT")
	button.TargetWarning.text:SetFontObject("GameFontNormalSmall")
	button.WidgetGroup:AddChild(button.TargetWarning)
	
	button.NeverSpell = AceGUI:Create("CheckBox")
	button.NeverSpell:SetWidth(70)
	button.NeverSpell:SetLabel("Never")
	button.NeverSpell.text:SetJustifyH("LEFT")
	button.NeverSpell.text:SetFontObject("GameFontNormalSmall")
	button.WidgetGroup:AddChild(button.NeverSpell)
	
	button.TrackedSpell = AceGUI:Create("CheckBox")
	button.TrackedSpell:SetWidth(70)
	button.TrackedSpell:SetLabel("Tracked")
	button.TrackedSpell.text:SetJustifyH("LEFT")
	button.TrackedSpell.text:SetFontObject("GameFontNormalSmall")
	button.WidgetGroup:AddChild(button.TrackedSpell)
	
	button.CDSound = AceGUI:Create("CheckBox")
	button.CDSound:SetWidth(80)
	button.CDSound:SetLabel("CD sound")
	button.CDSound.text:SetJustifyH("LEFT")
	button.CDSound.text:SetFontObject("GameFontNormalSmall")
	button.WidgetGroup:AddChild(button.CDSound)
	
	button.Setup = AceGUI:Create("Icon")
	button.Setup:SetImage(mediaFolder.."icons\\config")
	button.Setup:SetImageSize(20, 20)
	button.Setup:SetWidth(40)
	button.Setup:SetHeight(20)
	button.Setup.image:SetPoint("TOP", 0, 0)
	button.WidgetGroup:AddChild(button.Setup)
	
	button.Delete = AceGUI:Create("Icon")
	button.Delete:SetImage(mediaFolder.."marks\\7")
	button.Delete:SetImageSize(20, 20)
	button.Delete:SetWidth(20)
	button.Delete:SetHeight(20)
	button.Delete.image:SetPoint("TOP", 0, 0)
	button.Delete.frame:SetScript("OnEnter",function()
		GameTooltip:SetOwner(button.Delete.frame, "ANCHOR_BOTTOMRIGHT", 0, 0)
		GameTooltip:AddLine("Delete", 1, 0, 0)
		GameTooltip:Show()
	end)
	button.Delete.frame:SetScript("OnLeave",function()
		GameTooltip:Hide()
	end)
	button.WidgetGroup:AddChild(button.Delete)
		
	button.WidgetGroup.content:Hide()
		
	button:Show()
	return button
end
	
function npcInfoPanelMixin:HideNPCInfoPanel(from)
	if from == nil then from = 1 end
	if myNPCInfoPanelFrameBase.header and from == 1 then
		myNPCInfoPanelFrameBase.header:Hide()
		myNPCInfoPanelFrameBase.header:SetScript("OnUpdate", nil)
		myNPCInfoPanelFrameBase:Hide()
	end
	for i = from, #myNPCInfoPanelFrameBase.panel do
		myNPCInfoPanelFrameBase.panel[i]:Hide()
		myNPCInfoPanelFrameBase.panel[i]:SetScript("OnUpdate", nil)
	end
end

function npcInfoPanelMixin:ShowNPCInfoPanel(npc_ID, mapID)
	npcInfoPanelMixin:HideNPCInfoPanel()
	--if not npc_ID then return end
	local map_ID = L.MythicNPCData[0][npc_ID] and 0 or mapID
	if not map_ID then return end
		
	local buttonIndex = 1
	
	local function PanelOnEnter(self)
		self:GetParent().DescriptionGroup:Hide()
		self:GetParent().WidgetGroup.content:Show()
	end
	
	local function PanelOnLeave(self)
		if self:GetParent():IsMouseOver() then
			self:GetParent().DescriptionGroup:Hide()
			self:GetParent().WidgetGroup.content:Show()
		else
			self:GetParent().DescriptionGroup:Show()
			self:GetParent().WidgetGroup.content:Hide()
		end
	end
	
	local function PanelOnUpdate(self, elapsed)
		self.elapsed = (self.elapsed or 0) + elapsed
		if self.elapsed < .5 then return end
		if self:GetParent().SoundDropDown and self:GetParent().SoundDropDown.open then return end
		if self:GetParent().PredictedBox and self:GetParent().PredictedBox.editbox:HasFocus() then return end
		if self:GetParent().DescEditBox and self:GetParent().DescEditBox.editbox:HasFocus() then return end
	
		if self:IsMouseOver() then
			if self:GetParent().DescriptionGroup:IsShown() then
				self:GetParent().DescriptionGroup:Hide()
				self:GetParent().WidgetGroup.content:Show()
			end
		elseif not self:GetParent().DescriptionGroup:IsShown() then
			self:GetParent().DescriptionGroup:Show()
			self:GetParent().WidgetGroup.content:Hide()
		end
	end
	
		
	if L.MythicNPCData[map_ID][npc_ID] then
		myNPCInfoPanelFrameBase.header = myNPCInfoPanelFrameBase.header or CreateHeaderPanel("ButtonNpc")
		myNPCInfoPanelFrameBase.header:SetPoint("TOPLEFT", myNPCInfoPanelFrameBase, 5, -5)
		myNPCInfoPanelFrameBase.header:Show()
		myNPCInfoPanelFrameBase.header.DescriptionGroup:Show()
		myNPCInfoPanelFrameBase.header.WidgetGroup.content:Hide()
		myNPCInfoPanelFrameBase.header.Color:SetDisabled(not L.MythicNPCData[map_ID][npc_ID].colored)
		myNPCInfoPanelFrameBase.header.shadow:SetScript("OnUpdate", PanelOnUpdate)
		if L.MythicNPCData[map_ID][npc_ID].colored then
			myNPCInfoPanelFrameBase.header.npcName:SetTextColor(unpack(L.MythicNPCData[map_ID][npc_ID].color))
		else
			myNPCInfoPanelFrameBase.header.npcName:SetTextColor(1, 1, 1, 1)
		end
		myNPCInfoPanelFrameBase.header.npcName:SetText(L.MythicNPCData[map_ID][npc_ID].destName)
		myNPCInfoPanelFrameBase.header.npcName:Show()
		
		myNPCInfoPanelFrameBase.header.DescEditBox:SetText(L.MythicNPCData[map_ID][npc_ID].desc)
		myNPCInfoPanelFrameBase.header.DescEditBox:SetCallback("OnEnterPressed", function(widget, callbackName, value)
			if value ~= nil then
				L.MythicNPCData[map_ID][npc_ID].desc = value
			else
				L.MythicNPCData[map_ID][npc_ID].desc = nil
			end
			LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
			namePlateMixin:UpdateNamePlateByOptions("bars")
		end)
		
		myNPCInfoPanelFrameBase.header.Colored:SetValue(L.MythicNPCData[map_ID][npc_ID].colored)
		myNPCInfoPanelFrameBase.header.Colored:SetCallback("OnValueChanged", function(widget, callbackName, value)
			L.MythicNPCData[map_ID][npc_ID].colored = value and value or nil
			myNPCInfoPanelFrameBase.header.Color:SetDisabled(not L.MythicNPCData[map_ID][npc_ID].colored)
			if L.MythicNPCData[map_ID][npc_ID].colored then
				L.MythicNPCData[map_ID][npc_ID].color = {.7, .1, .9, 1}
				myNPCInfoPanelFrameBase.header.npcName:SetTextColor(unpack(L.MythicNPCData[map_ID][npc_ID].color))
			else
				L.MythicNPCData[map_ID][npc_ID].color = nil
				myNPCInfoPanelFrameBase.header.npcName:SetTextColor(1, 1, 1, 1)
			end
			LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
			namePlateMixin:UpdateNamePlateByOptions("bars")
		end)
		
		if L.MythicNPCData[map_ID][npc_ID].colored then
			myNPCInfoPanelFrameBase.header.Color:SetColor(unpack(L.MythicNPCData[map_ID][npc_ID].color))
		else
			myNPCInfoPanelFrameBase.header.Color:SetColor(1, 1, 1, 1)
		end
		myNPCInfoPanelFrameBase.header.Color:SetCallback("OnValueChanged", function(widget, callbackName, r, g, b, a)
			L.MythicNPCData[map_ID][npc_ID].color = {r, g, b, a}
			myNPCInfoPanelFrameBase.header.npcName:SetTextColor(unpack(L.MythicNPCData[map_ID][npc_ID].color))
			LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
			namePlateMixin:UpdateNamePlateByOptions("bars")
		end)
		
		
		for spellID in pairs(L.MythicNPCData[map_ID][npc_ID]) do
			if tonumber(spellID) then
				local spellName, _, spellTexture = GetSpellInfo(spellID)
				if spellName then
					myNPCInfoPanelFrameBase.panel[buttonIndex] = myNPCInfoPanelFrameBase.panel[buttonIndex] or CreateSpellPanel("Button"..buttonIndex)
					myNPCInfoPanelFrameBase.panel[buttonIndex]:SetSize(360, 75)				
					myNPCInfoPanelFrameBase.panel[buttonIndex].desc:SetText(nil)
					myNPCInfoPanelFrameBase.panel[buttonIndex].icon:SetTexture(spellTexture)
					myNPCInfoPanelFrameBase.panel[buttonIndex]:Show()
					myNPCInfoPanelFrameBase.panel[buttonIndex].DescriptionGroup:Show()
					myNPCInfoPanelFrameBase.panel[buttonIndex].WidgetGroup.content:Hide()
					myNPCInfoPanelFrameBase.panel[buttonIndex].shadow:SetScript("OnUpdate", PanelOnUpdate)
								
					local spell = Spell:CreateFromSpellID(spellID)
					if (not spell:IsSpellEmpty()) then
						spell.buttonIndex = buttonIndex
						spell:ContinueOnSpellLoad(function()
							myNPCInfoPanelFrameBase.panel[spell.buttonIndex].desc:SetText("|cff00bbff"..spellName.."|r\n"..spell:GetSpellDescription())
						end)
					end

					if L.MythicNPCData[map_ID][npc_ID][spellID].notInterruptible or L.MythicNPCData[map_ID][npc_ID][spellID].notInterruptible == nil then
						myNPCInfoPanelFrameBase.panel[buttonIndex].desc:SetTextColor(1, 1, 1, 1)
					else
						myNPCInfoPanelFrameBase.panel[buttonIndex].desc:SetTextColor(0, 1, 0, 1)
					end
				
					myNPCInfoPanelFrameBase.panel[buttonIndex].Slider:SetValue(L.MythicNPCData[map_ID][npc_ID][spellID].size or 1)
					myNPCInfoPanelFrameBase.panel[buttonIndex].Slider:SetCallback("OnValueChanged", function(widget, callbackName, value)
						if value ~= 1 then
							L.MythicNPCData[map_ID][npc_ID][spellID].size = value
						else
							L.MythicNPCData[map_ID][npc_ID][spellID].size = nil
						end
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
						namePlateMixin:UpdateNamePlateByOptions("nameplate")
					end)
					
					myNPCInfoPanelFrameBase.panel[buttonIndex].SoundDropDown:SetValue(GetKeyForValue(Media.MediaList["sound"], L.MythicNPCData[map_ID][npc_ID][spellID].sound))
					myNPCInfoPanelFrameBase.panel[buttonIndex].SoundDropDown:SetCallback("OnValueChanged",function(widget,callbackName,key)
						local sound = GetValueForKey( Media.MediaList["sound"], key)
						
						if sound == nil or sound == "None" then
							L.MythicNPCData[map_ID][npc_ID][spellID].sound = nil
						else
							L.MythicNPCData[map_ID][npc_ID][spellID].sound = val
						end
						
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
					end)
					
					myNPCInfoPanelFrameBase.panel[buttonIndex].PredictedBox:SetText(L.MythicNPCData[map_ID][npc_ID][spellID].trackedtime)
					myNPCInfoPanelFrameBase.panel[buttonIndex].PredictedBox:SetCallback("OnEnterPressed", function(widget, callbackName, value)
						L.MythicNPCData[map_ID][npc_ID][spellID].trackedtime = tonumber(value)
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
						namePlateMixin:UpdateNamePlateByOptions("nameplate")
					end)
												
					myNPCInfoPanelFrameBase.panel[buttonIndex].TargetWarning:SetValue(L.MythicNPCData[map_ID][npc_ID][spellID].targetme)
					myNPCInfoPanelFrameBase.panel[buttonIndex].TargetWarning:SetCallback("OnValueChanged", function(widget, callbackName, value)
						L.MythicNPCData[map_ID][npc_ID][spellID].targetme = value and value or nil
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
					end)
					
					myNPCInfoPanelFrameBase.panel[buttonIndex].NeverSpell:SetValue(L.MythicNPCData[map_ID][npc_ID][spellID].never)
					myNPCInfoPanelFrameBase.panel[buttonIndex].NeverSpell:SetCallback("OnValueChanged", function(widget, callbackName, value)
						L.MythicNPCData[map_ID][npc_ID][spellID].never = value and value or nil
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
						namePlateMixin:UpdateNamePlateByOptions("nameplate")
					end)
					
					myNPCInfoPanelFrameBase.panel[buttonIndex].TrackedSpell:SetValue(L.MythicNPCData[map_ID][npc_ID][spellID].tracked)
					myNPCInfoPanelFrameBase.panel[buttonIndex].TrackedSpell:SetCallback("OnValueChanged", function(widget, callbackName, value)
						L.MythicNPCData[map_ID][npc_ID][spellID].tracked = value and value or nil
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
						namePlateMixin:UpdateNamePlateByOptions("nameplate")
					end)
					
					myNPCInfoPanelFrameBase.panel[buttonIndex].CDSound:SetValue(L.MythicNPCData[map_ID][npc_ID][spellID].CDsound)
					myNPCInfoPanelFrameBase.panel[buttonIndex].CDSound:SetCallback("OnValueChanged", function(widget, callbackName, value)
						L.MythicNPCData[map_ID][npc_ID][spellID].CDsound = value and value or nil
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
					end)
					
					myNPCInfoPanelFrameBase.panel[buttonIndex].Setup:SetCallback("OnClick",function(widget, callbackName, button)
						LibStub('AceConfigDialog-3.0'):SelectGroup("Nameplates", "NPC_DB")
						LibStub('AceConfigDialog-3.0'):Open("Nameplates")
						L.UpdateNPCDatabase(map_ID, npc_ID, spellID)
					end)
					
					myNPCInfoPanelFrameBase.panel[buttonIndex].Delete:SetCallback("OnClick",function(widget, callbackName, button)
						L.MythicNPCData[map_ID][npc_ID][spellID] = nil;
						L.UpdateNPCDatabase(map_ID, npc_ID, nil)
						LibStub("AceConfigRegistry-3.0"):NotifyChange("Nameplates")
						npcInfoPanelMixin:ShowNPCInfoPanel(npc_ID, map_ID)
					end)
							
					if buttonIndex == 1 then
						myNPCInfoPanelFrameBase.panel[buttonIndex]:SetPoint("TOPLEFT", myNPCInfoPanelFrameBase.header, "BOTTOMLEFT", 0, -20)
					else
						myNPCInfoPanelFrameBase.panel[buttonIndex]:SetPoint("TOPLEFT", myNPCInfoPanelFrameBase.panel[buttonIndex - 1], "BOTTOMLEFT", 0, -20)
					end
					
					buttonIndex = buttonIndex + 1
					myNPCInfoPanelFrameBase:Show()
				end	
			end
		end
		
		npcInfoPanelMixin:HideNPCInfoPanel(buttonIndex)
	end
end

function npcInfoPanelMixin:OnEvent(event, ...)
	if ( event == "ADDON_LOADED" ) then
		local name = ...
		if name == _ then
			myNPCInfoPanelFrameBase:SetFrameLevel(1)
			myNPCInfoPanelFrameBase:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 5, -5)
			myNPCInfoPanelFrameBase:SetSize(100, 100)
			myNPCInfoPanelFrameBase:SetScale(1)
			myNPCInfoPanelFrameBase.panel = {}
			myNPCInfoPanelFrameBase:Hide()
			npcInfoPanelMixin:HideNPCInfoPanel()
			
			if IsAddOnLoaded(_) then
				self:UnregisterEvent('ADDON_LOADED')
				self.ADDON_LOADED = nil
			end
		end
	end
end

function npcInfoPanelMixin:OnLoad()
	for _, event in pairs({
		"ADDON_LOADED",
	}) do self:RegisterEvent(event) end
end
